@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtNetwork module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 //#define QHOSTINFO_DEBUG
 
@@ -43,6 +7,7 @@
 #include "qhostinfo_p.h"
 #include <qplatformdefs.h>
 
+#include "QtCore/qapplicationstatic.h"
 #include "QtCore/qscopedpointer.h"
 #include <qabstracteventdispatcher.h>
 #include <qcoreapplication.h>
@@ -51,7 +16,6 @@
 #include <qstringlist.h>
 #include <qthread.h>
 #include <qurl.h>
-#include <private/qnetworksession_p.h>
 
 #include <algorithm>
 
@@ -59,7 +23,7 @@
 #  include <unistd.h>
 #  include <netdb.h>
 #  include <netinet/in.h>
-#  if defined(AI_ADDRCONFIG)
+#  if defined(AI_ADDRCONFIG) && !defined(Q_OS_WASM)
 #    define Q_ADDRCONFIG          AI_ADDRCONFIG
 #  endif
 #elif defined Q_OS_WIN
@@ -70,9 +34,11 @@
 
 QT_BEGIN_NAMESPACE
 
+using namespace Qt::StringLiterals;
+
 //#define QHOSTINFO_DEBUG
 
-Q_GLOBAL_STATIC(QHostInfoLookupManager, theHostInfoLookupManager)
+QT_IMPL_METATYPE_EXTERN(QHostInfo)
 
 namespace {
 struct ToBeLookedUpEquals {
@@ -102,8 +68,20 @@ std::pair<OutputIt1, OutputIt2> separate_if(InputIt first, InputIt last, OutputI
     return std::make_pair(dest1, dest2);
 }
 
+Q_APPLICATION_STATIC(QHostInfoLookupManager, theHostInfoLookupManager)
+
 }
 
+QHostInfoResult::QHostInfoResult(const QObject *receiver, QtPrivate::SlotObjUniquePtr slot)
+    : receiver{receiver ? receiver : this}, slotObj{std::move(slot)}
+{
+    Q_ASSERT(this->receiver);
+    moveToThread(this->receiver->thread());
+}
+
+QHostInfoResult::~QHostInfoResult()
+    = default;
+
 /*
     The calling thread is likely the one that executes the lookup via
     QHostInfoRunnable. Unless we operate with a queued connection already,
@@ -112,8 +90,8 @@ std::pair<OutputIt1, OutputIt2> separate_if(InputIt first, InputIt last, OutputI
     the thread that made the call to lookupHost. That QHostInfoResult object
     then calls the user code in the correct thread.
 
-    The 'result' object deletes itself (via deleteLater) when the metacall
-    event is received.
+    The 'result' object deletes itself (via deleteLater) when
+    finalizePostResultsReady is called.
 */
 void QHostInfoResult::postResultsReady(const QHostInfo &info)
 {
@@ -123,53 +101,33 @@ void QHostInfoResult::postResultsReady(const QHostInfo &info)
         return;
     }
     // we used to have a context object, but it's already destroyed
-    if (withContextObject && !receiver)
+    if (!receiver)
         return;
 
-    static const int signal_index = []() -> int {
-        auto senderMetaObject = &QHostInfoResult::staticMetaObject;
-        auto signal = &QHostInfoResult::resultsReady;
-        int signal_index = -1;
-        void *args[] = { &signal_index, &signal };
-        senderMetaObject->static_metacall(QMetaObject::IndexOfMethod, 0, args);
-        return signal_index + QMetaObjectPrivate::signalOffset(senderMetaObject);
-    }();
-
     // a long-living version of this
     auto result = new QHostInfoResult(this);
     Q_CHECK_PTR(result);
 
-    const int nargs = 2;
-    auto metaCallEvent = new QMetaCallEvent(slotObj, nullptr, signal_index, nargs);
-    Q_CHECK_PTR(metaCallEvent);
-    void **args = metaCallEvent->args();
-    int *types = metaCallEvent->types();
-    types[0] = QMetaType::type("void");
-    types[1] = QMetaType::type("QHostInfo");
-    args[0] = nullptr;
-    args[1] = QMetaType::create(types[1], &info);
-    Q_CHECK_PTR(args[1]);
-    qApp->postEvent(result, metaCallEvent);
+    QMetaObject::invokeMethod(result,
+                              &QHostInfoResult::finalizePostResultsReady,
+                              Qt::QueuedConnection,
+                              info);
 }
 
 /*
-    Receives the event posted by postResultsReady, and calls the functor.
+    Receives the info from postResultsReady, and calls the functor.
 */
-bool QHostInfoResult::event(QEvent *event)
+void QHostInfoResult::finalizePostResultsReady(const QHostInfo &info)
 {
-    if (event->type() == QEvent::MetaCall) {
-        Q_ASSERT(slotObj);
-        auto metaCallEvent = static_cast<QMetaCallEvent *>(event);
-        auto args = metaCallEvent->args();
-        // we didn't have a context object, or it's still alive
-        if (!withContextObject || receiver)
-            slotObj->call(const_cast<QObject*>(receiver.data()), args);
-        slotObj->destroyIfLastRef();
+    Q_ASSERT(slotObj);
 
-        deleteLater();
-        return true;
+    // we used to have a context object, but it's already destroyed
+    if (receiver) {
+        void *args[] = { nullptr, const_cast<QHostInfo *>(&info) };
+        slotObj->call(const_cast<QObject *>(receiver.data()), args);
     }
-    return QObject::event(event);
+
+    deleteLater();
 }
 
 /*!
@@ -216,7 +174,7 @@ bool QHostInfoResult::event(QEvent *event)
     QHostInfo::localHostName() function.
 
     QHostInfo uses the mechanisms provided by the operating system
-    to perform the lookup. As per {https://tools.ietf.org/html/rfc6724}{RFC 6724}
+    to perform the lookup. As per \l {RFC 6724}
     there is no guarantee that all IP addresses registered for a domain or
     host will be returned.
 
@@ -227,13 +185,12 @@ bool QHostInfoResult::event(QEvent *event)
     \note Since Qt 4.6.3 QHostInfo is using a small internal 60 second DNS cache
     for performance improvements.
 
-    \sa QAbstractSocket, {http://www.rfc-editor.org/rfc/rfc3492.txt}{RFC 3492},
-    {https://tools.ietf.org/html/rfc6724}{RFC 6724}
+    \sa QAbstractSocket, {RFC 3492}, {RFC 6724}
 */
 
 static int nextId()
 {
-    static QBasicAtomicInt counter = Q_BASIC_ATOMIC_INITIALIZER(0);
+    Q_CONSTINIT static QBasicAtomicInt counter = Q_BASIC_ATOMIC_INITIALIZER(0);
     return 1 + counter.fetchAndAddRelaxed(1);
 }
 
@@ -264,11 +221,18 @@ static int nextId()
     \note There is no guarantee on the order the signals will be emitted
     if you start multiple requests with lookupHost().
 
+    \note In Qt versions prior to 6.7, this function took \a receiver as
+    (non-const) \c{QObject*}.
+
     \sa abortHostLookup(), addresses(), error(), fromName()
 */
-int QHostInfo::lookupHost(const QString &name, QObject *receiver, const char *member)
+int QHostInfo::lookupHost(const QString &name, const QObject *receiver, const char *member)
 {
-    return QHostInfoPrivate::lookupHostImpl(name, receiver, nullptr, member);
+    if (!receiver || !member) {
+        qWarning("QHostInfo::lookupHost: both the receiver and the member to invoke must be non-null");
+        return -1;
+    }
+    return QHostInfo::lookupHostImpl(name, receiver, nullptr, member);
 }
 
 /*!
@@ -293,7 +257,7 @@ int QHostInfo::lookupHost(const QString &name, QObject *receiver, const char *me
 */
 
 /*!
-    \fn template<typename Functor> int QHostInfo::lookupHost(const QString &name, Functor functor)
+    \fn template<typename Functor> int QHostInfo::lookupHost(const QString &name, Functor &&functor)
 
     \since 5.9
 
@@ -377,32 +341,16 @@ QHostInfo QHostInfo::fromName(const QString &name)
     qDebug("QHostInfo::fromName(\"%s\")",name.toLatin1().constData());
 #endif
 
+#ifdef Q_OS_WASM
+    return QHostInfoAgent::lookup(name);
+#else
     QHostInfo hostInfo = QHostInfoAgent::fromName(name);
     QHostInfoLookupManager* manager = theHostInfoLookupManager();
     manager->cache.put(name, hostInfo);
     return hostInfo;
+#endif
 }
 
-#ifndef QT_NO_BEARERMANAGEMENT
-QHostInfo QHostInfoPrivate::fromName(const QString &name, QSharedPointer<QNetworkSession> session)
-{
-#if defined QHOSTINFO_DEBUG
-    qDebug("QHostInfoPrivate::fromName(\"%s\") with session %p",name.toLatin1().constData(), session.data());
-#endif
-
-    QHostInfo hostInfo = QHostInfoAgent::fromName(name, session);
-    QHostInfoLookupManager* manager = theHostInfoLookupManager();
-    manager->cache.put(name, hostInfo);
-    return hostInfo;
-}
-#endif
-
-#ifndef QT_NO_BEARERMANAGEMENT
-QHostInfo QHostInfoAgent::fromName(const QString &hostName, QSharedPointer<QNetworkSession>)
-{
-    return QHostInfoAgent::fromName(hostName);
-}
-#endif
 
 QHostInfo QHostInfoAgent::reverseLookup(const QHostAddress &address)
 {
@@ -410,7 +358,7 @@ QHostInfo QHostInfoAgent::reverseLookup(const QHostAddress &address)
     // Reverse lookup
     sockaddr_in sa4;
     sockaddr_in6 sa6;
-    sockaddr *sa = 0;
+    sockaddr *sa = nullptr;
     QT_SOCKLEN_T saSize;
     if (address.protocol() == QAbstractSocket::IPv4Protocol) {
         sa = reinterpret_cast<sockaddr *>(&sa4);
@@ -455,7 +403,7 @@ QHostInfo QHostInfoAgent::lookup(const QString &hostName)
         return results;
     }
 
-    addrinfo *res = 0;
+    addrinfo *res = nullptr;
     struct addrinfo hints;
     memset(&hints, 0, sizeof(hints));
     hints.ai_family = PF_UNSPEC;
@@ -549,7 +497,7 @@ QHostInfo QHostInfoAgent::lookup(const QString &hostName)
         QString tmp;
         QList<QHostAddress> addresses = results.addresses();
         for (int i = 0; i < addresses.count(); ++i) {
-            if (i != 0) tmp += QLatin1String(", ");
+            if (i != 0) tmp += ", "_L1;
             tmp += addresses.at(i).toString();
         }
         qDebug("QHostInfoAgent::fromName(): found %i entries for \"%s\": {%s}",
@@ -612,10 +560,11 @@ QHostInfo::QHostInfo(const QHostInfo &other)
 */
 QHostInfo &QHostInfo::operator=(const QHostInfo &other)
 {
-    if (d_ptr)
-        *d_ptr = *other.d_ptr;
-    else
-        d_ptr = new QHostInfoPrivate(*other.d_ptr);
+    if (this == &other)
+      return *this;
+
+    Q_ASSERT(d_ptr && other.d_ptr);
+    *d_ptr = *other.d_ptr;
     return *this;
 }
 
@@ -776,32 +725,29 @@ QString QHostInfo::localHostName()
     \sa hostName()
 */
 
-// ### Qt 6 merge with function below
-int QHostInfo::lookupHostImpl(const QString &name,
-                              const QObject *receiver,
-                              QtPrivate::QSlotObjectBase *slotObj)
-{
-    return QHostInfoPrivate::lookupHostImpl(name, receiver, slotObj, nullptr);
-}
-/*
+/*!
+    \internal
     Called by the various lookupHost overloads to perform the lookup.
 
-    Signals either the functor encapuslated in the \a slotObj in the context
+    Signals either the functor encapuslated in the \a slotObjRaw in the context
     of \a receiver, or the \a member slot of the \a receiver.
 
-    \a receiver might be the nullptr, but only if a \a slotObj is provided.
+    \a receiver might be the nullptr, but only if a \a slotObjRaw is provided.
 */
-int QHostInfoPrivate::lookupHostImpl(const QString &name,
-                                     const QObject *receiver,
-                                     QtPrivate::QSlotObjectBase *slotObj,
-                                     const char *member)
+int QHostInfo::lookupHostImpl(const QString &name,
+                              const QObject *receiver,
+                              QtPrivate::QSlotObjectBase *slotObjRaw,
+                              const char *member)
 {
+    QtPrivate::SlotObjUniquePtr slotObj{slotObjRaw};
 #if defined QHOSTINFO_DEBUG
-    qDebug("QHostInfoPrivate::lookupHostImpl(\"%s\", %p, %p, %s)",
-           name.toLatin1().constData(), receiver, slotObj, member ? member + 1 : 0);
+    qDebug("QHostInfo::lookupHostImpl(\"%s\", %p, %p, %s)",
+           name.toLatin1().constData(), receiver, slotObj.get(), member ? member + 1 : 0);
 #endif
     Q_ASSERT(!member != !slotObj); // one of these must be set, but not both
     Q_ASSERT(receiver || slotObj);
+    Q_ASSERT(!member || receiver); // if member is set, also is receiver
+    const bool isUsingStringBasedSlot = static_cast<bool>(member);
 
     if (!QAbstractEventDispatcher::instance(QThread::currentThread())) {
         qWarning("QHostInfo::lookupHost() called with no event dispatcher");
@@ -817,15 +763,31 @@ int QHostInfoPrivate::lookupHostImpl(const QString &name,
         hostInfo.setError(QHostInfo::HostNotFound);
         hostInfo.setErrorString(QCoreApplication::translate("QHostInfo", "No host name given"));
 
-        QHostInfoResult result(receiver, slotObj);
-        if (receiver && member)
+        QHostInfoResult result(receiver, std::move(slotObj));
+        if (isUsingStringBasedSlot) {
             QObject::connect(&result, SIGNAL(resultsReady(QHostInfo)),
                             receiver, member, Qt::QueuedConnection);
+        }
         result.postResultsReady(hostInfo);
 
         return id;
     }
 
+#ifdef Q_OS_WASM
+    // Resolve the host name directly without using a thread or cache,
+    // since Emscripten's host lookup is fast. Emscripten maintains an internal
+    // mapping of hosts and addresses for the purposes of WebSocket socket
+    // tunnelling, and does not perform an actual host lookup.
+    QHostInfo hostInfo = QHostInfoAgent::lookup(name);
+    hostInfo.setLookupId(id);
+
+    QHostInfoResult result(receiver, std::move(slotObj));
+    if (isUsingStringBasedSlot) {
+        QObject::connect(&result, SIGNAL(resultsReady(QHostInfo)),
+                        receiver, member, Qt::QueuedConnection);
+    }
+    result.postResultsReady(hostInfo);
+#else
     QHostInfoLookupManager *manager = theHostInfoLookupManager();
 
     if (Q_LIKELY(manager)) {
@@ -836,32 +798,38 @@ int QHostInfoPrivate::lookupHostImpl(const QString &name,
             QHostInfo info = manager->cache.get(name, &valid);
             if (valid) {
                 info.setLookupId(id);
-                QHostInfoResult result(receiver, slotObj);
-                if (receiver && member)
+                QHostInfoResult result(receiver, std::move(slotObj));
+                if (isUsingStringBasedSlot) {
                     QObject::connect(&result, SIGNAL(resultsReady(QHostInfo)),
                                     receiver, member, Qt::QueuedConnection);
+                }
                 result.postResultsReady(info);
                 return id;
             }
         }
 
         // cache is not enabled or it was not in the cache, do normal lookup
-        QHostInfoRunnable *runnable = new QHostInfoRunnable(name, id, receiver, slotObj);
-        if (receiver && member)
+        QHostInfoRunnable *runnable = new QHostInfoRunnable(name, id, receiver, std::move(slotObj));
+        if (isUsingStringBasedSlot) {
             QObject::connect(&runnable->resultEmitter, SIGNAL(resultsReady(QHostInfo)),
                                 receiver, member, Qt::QueuedConnection);
+        }
         manager->scheduleLookup(runnable);
     }
+#endif // Q_OS_WASM
     return id;
 }
 
 QHostInfoRunnable::QHostInfoRunnable(const QString &hn, int i, const QObject *receiver,
-                                     QtPrivate::QSlotObjectBase *slotObj) :
-    toBeLookedUp(hn), id(i), resultEmitter(receiver, slotObj)
+                                     QtPrivate::SlotObjUniquePtr slotObj)
+    : toBeLookedUp{hn}, id{i}, resultEmitter{receiver, std::move(slotObj)}
 {
     setAutoDelete(true);
 }
 
+QHostInfoRunnable::~QHostInfoRunnable()
+    = default;
+
 // the QHostInfoLookupManager will at some point call this via a QThreadPool
 void QHostInfoRunnable::run()
 {
@@ -971,7 +939,7 @@ void QHostInfoLookupManager::rescheduleWithMutexHeld()
 
     if (!finishedLookups.isEmpty()) {
         // remove ID from aborted if it is in there
-        for (int i = 0; i < finishedLookups.length(); i++) {
+        for (int i = 0; i < finishedLookups.size(); i++) {
            abortedLookups.removeAll(finishedLookups.at(i)->id);
         }
 
@@ -999,7 +967,7 @@ void QHostInfoLookupManager::rescheduleWithMutexHeld()
                                        isAlreadyRunning).second,
                            scheduledLookups.end());
 
-    const int availableThreads = threadPool.maxThreadCount() - currentLookups.size();
+    const int availableThreads = std::max(threadPool.maxThreadCount(), 1) - currentLookups.size();
     if (availableThreads > 0) {
         int readyToStartCount = qMin(availableThreads, scheduledLookups.size());
         auto it = scheduledLookups.begin();
@@ -1037,9 +1005,12 @@ void QHostInfoLookupManager::abortLookup(int id)
     if (wasDeleted)
         return;
 
+    if (id == -1)
+        return;
+
 #if QT_CONFIG(thread)
     // is postponed? delete and return
-    for (int i = 0; i < postponedLookups.length(); i++) {
+    for (int i = 0; i < postponedLookups.size(); i++) {
         if (postponedLookups.at(i)->id == id) {
             delete postponedLookups.takeAt(i);
             return;
@@ -1048,7 +1019,7 @@ void QHostInfoLookupManager::abortLookup(int id)
 #endif
 
     // is scheduled? delete and return
-    for (int i = 0; i < scheduledLookups.length(); i++) {
+    for (int i = 0; i < scheduledLookups.size(); i++) {
         if (scheduledLookups.at(i)->id == id) {
             delete scheduledLookups.takeAt(i);
             return;
@@ -1101,7 +1072,7 @@ QHostInfo qt_qhostinfo_lookup(const QString &name, QObject *receiver, const char
     }
 
     // was not in cache, trigger lookup
-    *id = QHostInfoPrivate::lookupHostImpl(name, receiver, nullptr, member);
+    *id = QHostInfo::lookupHostImpl(name, receiver, nullptr, member);
 
     // return empty response, valid==false
     return QHostInfo();
@@ -1183,3 +1154,6 @@ void QHostInfoCache::clear()
 }
 
 QT_END_NAMESPACE
+
+#include "moc_qhostinfo_p.cpp"
+#include "moc_qhostinfo.cpp"
