@@ -271,7 +271,7 @@ int QStatusBar::insertWidget(int index, QWidget *widget, int stretch)
         widget->hide();
 
     reformat();
-    if (!widget->isHidden() || !widget->testAttribute(Qt::WA_WState_ExplicitShowHide))
+    if (!QWidgetPrivate::get(widget)->isExplicitlyHidden())
         widget->show();
 
     return index;
@@ -332,7 +332,7 @@ int QStatusBar::insertPermanentWidget(int index, QWidget *widget, int stretch)
     d->items.insert(index, item);
 
     reformat();
-    if (!widget->isHidden() || !widget->testAttribute(Qt::WA_WState_ExplicitShowHide))
+    if (!QWidgetPrivate::get(widget)->isExplicitlyHidden())
         widget->show();
 
     return index;
@@ -494,7 +494,7 @@ void QStatusBar::showMessage(const QString &message, int timeout)
     if (timeout > 0) {
         if (!d->timer) {
             d->timer = new QTimer(this);
-            connect(d->timer, SIGNAL(timeout()), this, SLOT(clearMessage()));
+            connect(d->timer, &QTimer::timeout, this, &QStatusBar::clearMessage);
         }
         d->timer->start(timeout);
     } else if (d->timer) {
@@ -520,7 +520,7 @@ void QStatusBar::clearMessage()
     if (d->tempItem.isEmpty())
         return;
     if (d->timer) {
-        qDeleteInEventHandler(d->timer);
+        delete d->timer;
         d->timer = nullptr;
     }
     d->tempItem.clear();
